#!/usr/bin/env python
# coding: utf-8


# ESA (C) 2000-2022
#
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.



#inthist_utils.py

import matplotlib.pyplot as plt
import os
import glob
import numpy as np
from matplotlib.backends.backend_pdf import PdfPages
import math
import sys
import warnings



def load_files(path_to_file):
    '''
    Loads the temporary text files into numpy 1D arrays.
    
    Args:
        path_to_file: the path to one file.
        
    Output:
        data: the data as an array.
    '''

    try:
        data = np.loadtxt(path_to_file)
    except FileNotFoundError:
        raise FileNotFoundError('Could not open file {}. Quitting...'.format(path_to_file))
    
    return data


def return_text(file_info):
    '''
    Returns the basic text with the information passed from the file.

    Args:
        file_info: the information from the file.

    Output:
        text: the text ready and formatted.
    '''

    try:
        with open(file_info) as fi:
            lines = fi.readlines()
            text = ''
            for line in lines:
                text = text + line

        text = text.replace('\\d', '')
        text = text.replace('\\u2', '^2')
    except FileNotFoundError:
        warnings.warn('Could not open the given info file: {}.'.format(file_info))
        return ''
    
    os.remove(file_info)

    return text


def main(filename):
    '''
    Main function.

    Args:
        filename: the name of the output file.

    Output:
        none.
    '''
    
#    file_info = gather_info(in_file)
    text = return_text('info_tmp')
    bins_files = glob.glob('bins*tmp')
    hist_files = glob.glob('hist*tmp')

    bins_files.sort()
    hist_files.sort()
    n_files = len(hist_files)
    counts = 0
    n_pages = math.ceil((n_files + 1) / 6) # + 1 counts for the text box
    fig_count = 0

    with PdfPages(filename) as pdf:
        for page in range(0, n_pages):
            fig, axs = plt.subplots(2, 3)
            # Creates a dictionary to evaluate which subplots are used
            plot_dict = {(0, 0) : False, (0, 1) : False, (0, 2) : False,\
                    (1, 0) : False, (1, 1) : False, (1, 2) : False}
            fig.set_figheight(10)
            i1, i2 = 0, 0
            fig.set_figwidth(16)
            axs[i1, i2].axis('off')
            axs[i1, i2].text(0, 0.5, text, fontsize = 14)
            plot_dict.update({(0, 0) : True})

            for i in range(fig_count, n_files):
                hist_file = hist_files[i]
                bins_file = bins_files[i]
                hist = load_files(hist_file)
                bins = load_files(bins_file)
                fig_count = fig_count + 1

                # getting indeces
                if counts < 2 and i1 == 0:
                    i2 = counts + 1
                else:
                    i2 = counts - 2
                    i1 = 1

                # check if histogram can be done with the current file:
                try:
                    int_array = int(hist)
                    hist_flag = True
                except TypeError:
                    hist_flag = False
                if hist_flag:
                    e_text = 'Band: {}\n\n\n'.format(fig_count) +\
                            'All pixels have the same intensity.\n\n\nNo hisogram plotted.'
                    axs[i1, i2].text(0.05, 0.3, e_text, fontsize = 13)
                    axs[i1, i2].set_xticklabels({})
                    axs[i1, i2].set_yticklabels({})
                else:
                    width = abs(bins[0] - bins[1])
                    l_bin = bins[-1]
                    l_hhist = hist[-1] / 2

                    axs[i1, i2].bar(bins, hist, label = 'Band number {}'.format(fig_count), width = width, color = 'green', alpha = 0.3)
                    axs[i1, i2].set(xlabel = 'Count rate (ct/s)', ylabel = 'Number of bins')
                    axs[i1, i2].legend(loc = 'upper right')

                    xmin, xmax = axs[i1, i2].get_xlim()
                    size_arrow = abs(l_bin - xmax)

                    axs[i1, i2].arrow(x = l_bin, y = l_hhist, dx = width, color = 'green', dy = 0, length_includes_head = False)
                
                counts = counts + 1
                
                # upload the dictionary
                plot_key = (i1, i2)
                plot_dict.update({plot_key : True})
                if counts == 5:
                    counts = 0
                    break

            
            # using the dictionary to delete unused axes:
            for key in plot_dict.keys():
                if not plot_dict[key]:
                    i1, i2 = key
                    fig.delaxes(axs[i1, i2])
            
            fig.tight_layout()
            
            pdf.savefig()
            plt.close()


    # Cleaning up...
    for f in hist_files:
        os.remove(f)
    for f in bins_files:
        os.remove(f)


    return None


if __name__ == '__main__':
    filename = sys.argv[1]
    if filename.upper().endswith('.PS'):
        warnings.warn('Only format accepted is pdf.')
        filename = os.path.splitext(filename)[0] + '.pdf'

    main(filename)
